/*
 * Command.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/**
 * A class that encapsulates a simulator command. This class and its subclasses
 * implement the Command Pattern, as demonstrated in Gamma et al (page 236). A
 * REMOTE command knows how to operate on the Robot class to achieve a specific
 * action; the SimulationEngine class only needs to know the Command class, not
 * the details of how a particular Command is implemented.  This encapsulation
 * allows new Command classes to be added in the future to work with new robot 
 * language parsers.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 * @see SimulationEngine
 * @see CommandQueue
 */
public abstract class Command
{
    /**
     * Stores the line in the program responsible for generating this Command.
     */
    private int lineNumber;


    /**
     * Creates a new Command, associated with a given line number in the 
     * simulation program.
     *
     * @param lineNumber the source line for this command.
     */
    public Command(int lineNumber)
    {
        super();

        setSourceLine(lineNumber);
    }

    /**
     * Executes the Command's function.  The SimulationEngine will call this 
     * method in order to activate the Command and effect change on either the
     * Robot's state or the state of the simulation.
     *
     * @exception thrown if an error occurs attempting to execute the command.
     */
    public abstract void Execute() throws Exception;

    /**
     * Returns the line number in the source file which generated this Command.
     *
     * @return the line number of this Command in the source file.
     */
    public int getSourceLine()
    {
        return lineNumber;
    }

    /**
     * Sets the line number in the source file which generated this Command.
     *
     * @param lineNumber the line number of this Command in the source file.
     */
    public void setSourceLine(int lineNumber)
    {
        this.lineNumber = lineNumber;
    }
}