/*
 * CommandParser.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.*;
import java.util.*;

/**
 * An abstract class to encapsulate the functionality required to parse a
 * simulation file and extract commands to simulate the robot's actions.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 *
 * @see CommandQueue
 * @see Command
 */
public abstract class CommandParser
{
    /**
     * Private constructor to prevent direct instantiation. Use the method
     * <code>loadParser</code> to load a specific CommandParser instance.
     */
    protected CommandParser()
    {
        super();
    }

    /**
     * Returns a CommandQueue of Commands parsed from the simulation file.
     *
     * @return the CommandQueue object of parsed Command objects.
     */
    public abstract CommandQueue getCommandQueue();

    /**
	 * Sets the Hashtable of names for the Robot's Degrees of freedom.
	 *
	 * @param names the Hashtable of names for the Robot's joints.
	 */
    public abstract void setNameTable(Hashtable names);

    /**
	 * Gets the Hashtable of valid names for the Robot's Degrees of freedom.
	 *
	 * @return the Hashtable of name for the Robot's joints.
	 */
    public abstract Hashtable getNameTable();

    /**
	 * Loads the simulation file using the specified Reader.
	 *
	 * @param r the Reader to parse for Commands.
	 * @exception CommandParserException indicates error parsing Reader.
	 */
    public abstract void loadSimulation(Reader r)
        throws CommandParserException;

    /**
     * Loads the CommandParser class specified by the given String.
     *
     * @param s the name of the CommandParser class to be loaded.
     * @return an instance of the requested CommandParser class.
     * @exception CommandParserException indicates an error loading the parser.
     */
    public static final CommandParser loadParser(String s)
        throws CommandParserException
    {
        Class aClass;
        CommandParser aCommandParser;

        try
        {
            aClass = Class.forName(s);
            aCommandParser = (CommandParser)aClass.newInstance();

            return aCommandParser;
        }
        catch (Exception e)
        {
            throw new CommandParserException(e.getMessage());
        }
    }
}