/*
 * DefaultCommandParser.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.*;
import java.util.*;

/**
 * A class to encapsulate the functionality required to properly parse a 
 * simulation file in order to simulate the robot's actions. This particular 
 * implementation parses files according to the SCORBASE-type definition. The
 * implementation of this class needs to be cleaned up.
 *
 * @author Brendon Wilson
 * @version alpha
 */
public class DefaultCommandParser extends CommandParser
{
    /** The FIFO queue of Commands parsed from the input. */
    private CommandQueue theCommandQueue;

    /** The command language definition resources. */
    private Hashtable theResourceTable;

    /** The table of names for the Robot's Degrees of freedom. */
    private Hashtable theNameTable;

    /** A default set of command language definitions and error messages. */
    private String[][] resources =
    {
        {"syntax.define", "define"},
        {"syntax.goto", "goto"},
        {"syntax.move", "move"},
        {"syntax.position", "pos"},
        {"syntax.xyz", "xyz"},
        {"syntax.joint", "joint"},
        {"syntax.set", "set"},
        {"syntax.speed", "speed"},
        {"syntax.wait", "wait"},
        {"syntax.home", "home"},
        {"syntax.end", "end"},
        {"errors.joint.undefined", "Undefined JOINT name."},
        {"errors.position.undefined", "Undefined POS name. You must DEFINE POS <name> before using GOTO POS <name>."},
        {"errors.xyz.undefined", "Undefined XYZ name. You must DEFINE XYZ <name> before using GOTO XYZ <name>."},
        {"errors.define", "Invalid DEFINE command. DEFINE must be specified in either the DEFINE POS <name> or DEFINE XYZ <name> <x> <y> <z> form."},
        {"errors.define.position", "Invalid DEFINE POS command. DEFINE POS must be followed by a string."},
        {"errors.define.xyz", "Invalid DEFINE XYZ command. DEFINE XYZ must be followed by a name and three numbers specifying an X,Y,Z position."},
        {"errors.goto", "Invalid GOTO command. GOTO must be specified in either the GOTO POS or GOTO XYZ form."},
        {"errors.goto.position", "Invalid GOTO POS command. GOTO POS must be followed by a name specifying a previously defined position."},
        {"errors.goto.xyz", "Invalid GOTO XYZ command. GOTO XYZ must be followed by a name specifying a previously defined xyz position."},
        {"errors.move", "Invalid MOVE command. MOVE must be specified in either the MOVE XYZ or MOVE JOINT form."},
        {"errors.move.xyz", "Invalid MOVE XYZ command. MOVE XYZ must be followed by three number specifying an X, Y, Z position."},
        {"errors.move.joint", "Invalid MOVE JOINT command. MOVE JOINT must be followed a joint number and a joint variable."},
        {"errors.wait", "Invalid WAIT command. Wait must be followed by a number specifying a time in milliseconds."},
        {"errors.set", "Invalid SET command. SET must be specifed as SET SPEED <number>."},
        {"errors.set.speed", "Invalid SET SPEED command. SET SPEED must be followed by a number between 1 and 10."}};


    /**
     * The default constructor which retreives the language properties, fills 
     * the resource table, and initializes the class variables.
     */
    protected DefaultCommandParser()
    {
        theCommandQueue = new CommandQueue();
        theResourceTable = new Hashtable();
        theNameTable = new Hashtable();

        fillResources();
    }

    /**
     * Fills up the resources using the hardcoded and soft-loaded resources.
     */
    private void fillResources()
    {
        try
        {
            ResourceBundle bundle = ResourceBundle.getBundle("CommandParserConfig");
            theResourceTable = new Hashtable();

            for (int i = 0; i < resources.length; i++)
            {
                String temp;

                try
                {
                    temp = bundle.getString(resources[i][0]);
                }
                catch (MissingResourceException mre)
                {
                    temp = resources[i][1];
                }

                theResourceTable.put(resources[i][0], temp);
            }
        }
        catch (MissingResourceException mre)
        {
            System.out.println("DefaultCommandParser: Unable to find language resources.");
        }
    }

    /**
     * Returns the CommandQueue of the command file.
     *
     * @return the CommandQueue object of parsed Commands.
     */
    public CommandQueue getCommandQueue()
    {
        return theCommandQueue;
    }

    /**
     * Sets the Hashtable of valid names for the Robot.
     *
     * @param theNameTable the Hashtable of name for the Robot's joints.
     */
    public void setNameTable (Hashtable theNameTable)
    {
        this.theNameTable = theNameTable;
    }

    /**
     * Gets the Hashtable of valid names for the Robot.
     *
     * @return the Hashtable of names for the Robot's joints.
     */
    public Hashtable getNameTable()
    {
        return theNameTable;
    }

    /**
     * Loads the simulation file by reading from the specified Reader.
     *
     * @param reader the Reader stream for simulation.
     * @exception CommandParserException thrown is cases of I/O error.
     */
    public void loadSimulation(Reader reader)
        throws CommandParserException
    {
        Hashtable theXYZTable = new Hashtable();
        Hashtable thePositionTable = new Hashtable();
        boolean doContinue = true;
        StreamTokenizer st = new StreamTokenizer(reader);
        Command currentCommand;
        theCommandQueue = new CommandQueue();

        st.slashSlashComments(true);
        st.slashStarComments(true);

        try
        {
            while (doContinue)
            {
                switch (st.nextToken())
                {
                    case StreamTokenizer.TT_WORD:
                    {
                        if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.define"))) {
                            if (st.nextToken() == StreamTokenizer.TT_WORD) {
                                if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.position"))) {
                                    if (st.nextToken() == StreamTokenizer.TT_WORD)
                                    {
                                        thePositionTable.put(st.sval, "");
                                        currentCommand =  new DefinePositionCommand(st.lineno(), st.sval);
                                        theCommandQueue.putCommand(currentCommand);
                                    }
                                    else
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.define.position"));
                                    }
                                } 
                                else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.xyz")))
                                {
                                    double x, y, z;
                                    String name;

                                    if (st.nextToken() == StreamTokenizer.TT_WORD)
                                    {
                                        name = st.sval;

                                        if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                        {
                                            x = st.nval;

                                            if (st.nextToken() == StreamTokenizer.TT_NUMBER) {
                                                y =  st.nval;

                                                if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                                {
                                                    z =  st.nval;
                                                    currentCommand = new DefineNamedXYZCommand(st.lineno(), name, x, y, z);
                                                    theCommandQueue.putCommand(currentCommand);
                                                    theXYZTable.put(name, name);
                                                }
                                                else
                                                {
                                                    throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.define.xyz"));
                                                }
                                            }
                                            else
                                            {
                                                throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.define.xyz"));
                                            }
                                        }
                                        else
                                        {
                                            throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.define.xyz"));
                                        }
                                    }
                                    else
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.define.xyz"));
                                    }
                                }
                                else
                                {
                                    throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.define"));
                                }
                            }
                        }
                        else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.goto")))
                        {
                            if (st.nextToken() == StreamTokenizer.TT_WORD)
                            {
                                if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.position")))
                                {
                                    if (st.nextToken() == StreamTokenizer.TT_WORD)
                                    {
                                        if (thePositionTable.get(st.sval) == null)
                                        {
                                            throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.position.undefined"));
                                        }

                                        currentCommand = new GotoPositionCommand(st.lineno(), st.sval);
                                        theCommandQueue.putCommand(currentCommand);
                                    }
                                    else
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.goto.position"));
                                    }
                                }
                                else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.xyz")))
                                {
                                    if (st.nextToken() == StreamTokenizer.TT_WORD)
                                    {
                                        if (theXYZTable.get(st.sval) == null)
                                        {
                                            throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.xyz.undefined"));
                                        }

                                        currentCommand =  new GotoNamedXYZCommand(st.lineno(), st.sval);
                                        theCommandQueue.putCommand(currentCommand);
                                    }
                                    else
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.goto.xyz"));
                                    }
                                }
                                else
                                {
                                    throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.goto"));
                                }
                            }
                        }
                        else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.move")))
                        {
                            if (st.nextToken() == StreamTokenizer.TT_WORD)
                            {
                                if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.xyz")))
                                {
                                    double x, y, z;

                                    if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                    {
                                        x = st.nval;

                                        if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                        {
                                            y = st.nval;

                                            if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                            {
                                                z = st.nval;
                                                currentCommand = new MoveXYZCommand(st.lineno(), x, y, z);
                                                theCommandQueue.putCommand(currentCommand);
                                            }
                                            else
                                            {
                                                throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.move.xyz"));
                                            }
                                        }
                                        else
                                        {
                                            throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.move.xyz"));
                                        }
                                    }
                                    else
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.move.xyz"));
                                    }
                                } else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.joint")))
                                {
                                    String joint;
                                    double value;

                                    if (st.nextToken() == StreamTokenizer.TT_WORD)
                                    {
                                        joint = st.sval.toUpperCase();

                                        if (theNameTable.get(joint) == null)
                                        {
                                            throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.joint.undefined"));
                                        }

                                        if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                        {
                                            value = st.nval;
                                            currentCommand =  new MoveJointCommand(st.lineno(), joint, value);
                                            theCommandQueue.putCommand(currentCommand);
                                        }
                                        else
                                        {
                                            throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.move.joint"));
                                        }
                                    }
                                    else
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.move.joint"));
                                    }
                                }
                                else
                                {
                                    String joint = st.sval.toUpperCase();
                                    double value;

                                    if (theNameTable.get(joint) == null)
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.joint.undefined"));
                                    }

                                    if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                    {
                                        value = st.nval;
                                        currentCommand = new MoveJointCommand(st.lineno(), joint, value);
                                        theCommandQueue.putCommand(currentCommand);
                                    }
                                    else
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.move.joint"));
                                    }
                                }
                            }
                            else
                            {
                                throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.move"));
                            }
                        }
                        else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.wait")))
                        {
                            if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                            {
                                currentCommand = new WaitCommand(st.lineno(), new Double(st.nval).longValue());
                                theCommandQueue.putCommand(currentCommand);
                            }
                            else
                            {
                                throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.wait"));
                            }
                        }
                        else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.set")))
                        {
                            if (st.nextToken() == StreamTokenizer.TT_WORD)
                            {
                                if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.speed")))
                                {
                                    if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                    {
                                        currentCommand =  new SpeedCommand(st.lineno(), st.nval);
                                        theCommandQueue.putCommand(currentCommand);
                                    }
                                    else
                                    {
                                        throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.set.speed"));
                                    }
                                }
                                else
                                {
                                    throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.set"));
                                }
                            }
                            else
                            {
                                throw new CommandParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.set"));
                            }
                        }
                        else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.home")))
                        {
                            currentCommand = new HomeCommand(st.lineno());
                            theCommandQueue.putCommand(currentCommand);
                        }
                        else if ((st.sval).equalsIgnoreCase((String)theResourceTable.get("syntax.end")))
                        {
                            doContinue = false;
                        }

                        break;
                    }

                    case StreamTokenizer.TT_EOF:
                    {
                        doContinue = false;
                        break;
                    }
                }
            }
        }
        catch (IOException ie)
        {
            throw new CommandParserException(ie.getMessage());
        }
        finally
        {
            try
            {
                reader.close();
            }
            catch (IOException ie2)
            {
                throw new CommandParserException(ie2.getMessage());
            }
        }
    }
}