/*
 * DefaultModelParser.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.*;
import java.net.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import com.sun.j3d.loaders.objectfile.ObjectFile;
import com.sun.j3d.loaders.ParsingErrorException;
import com.sun.j3d.loaders.IncorrectFormatException;
import com.sun.j3d.loaders.Scene;

//import com.sun.j3d.loaders.vrml97.VrmlLoader;


/**
 * A class to encapsulate the default functionality required to properly
 * parse a model file representing a 3D object.  This class acts as a wrapper 
 * for the uses the ObjectFile class, which reads in 3D files in the Wavefront
 * .obj file format.  At present the ObjectFile implementation provided by 
 * Sun is not capable of parsing color, appearance, and texture attributes 
 * from the .obj file, although it is assumed that this functionality will 
 * be provided in the future.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 *
 * @see ModelParser
 * @see ObjectFile
 */
public class DefaultModelParser extends ModelParser
{
    /** The 3D representation parsed from file. */
    private Scene model;

    /** Hashtable of language resources. */
    private Hashtable rsrcTable;

    /** Default language resources. */
    private String[][] resources =
    {
        {"errors.source.unspecified", "No source specified"},
        {"errors.model.empty", "The model is empty."}
    };


    /**
     * The default constructor which retrieves the language properties, 
     * fill the resource table, and initializes the class variables.
     */
    protected DefaultModelParser()
    {
        rsrcTable = new Hashtable();
        fillResources();
    }

    /**
     * Fills up the resources using the hardcoded and soft-loaded resources.
     */
    public void fillResources()
    {
        try
        {
            ResourceBundle r = ResourceBundle.getBundle("ModelParserConfig");
            rsrcTable = new Hashtable();

            for (int i = 0; i < resources.length; i++)
            {
                String temp;

                try
                {
                    temp = r.getString(resources[i][0]);
                }
                catch (MissingResourceException mre)
                {
                    temp = resources[i][1];
                }

                rsrcTable.put(resources[i][0], temp);
            }
        }
        catch (MissingResourceException mre)
        {
            System.out.println("DefaultModelParser:missing language resources.");
        }
    }

    /**
     * Reads in the specified configuration from a specified file.
     *
     * @param filename the name of the file to parse for 3D information.
     * @exception ModelParserException thrown if an error occurs.
     */
    public void read (String filename) throws ModelParserException
    {
        if (filename == null)
        {
            throw new ModelParserException((String)rsrcTable.get("errors.source.unspecified"));
        }
		
		try
		{
            // Use the ObjectFile loader
            int flags = ObjectFile.TRIANGULATE;
            ObjectFile f = new ObjectFile(flags, (float)(60*Math.PI/180.0));
            model = f.load("rbt/" + filename);

            //VrmlLoader loader = new VrmlLoader();
            //model = loader.load("rbt/" + filename);
        }
        catch (Exception e)
		{
            throw new ModelParserException(e.toString());
		}
	}

    /**
     * Returns a Group representation of a parsed file. This function requires
     * the model file has been parsed, otherwise resulting in an exception.
     *
     * @return Model the representation of the model file.
     * @exception ModelParserException indicates the model is empty.
     */
    public Group getModel() throws ModelParserException
    {
        if (model == null)
        {
            throw new ModelParserException((String)rsrcTable.get("errors.model.empty"));
        }
        else
        {
            return model.getSceneGroup();
        }
    }
}