/*
 * DefaultRobotParser.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.*;
import java.util.*;

/**
 * A class to encapsulate the default functionality required to properly
 * parse a robot configuration file, and construct the robot's internal
 * representation using instances of the Model and InvKin classes.
 *
 * @author Brendon Wilson
 * @version alpha
 */
public class DefaultRobotParser extends RobotParser
{
    /** Token identifier for no command. */
    private static final int NULL = 0;

    /** Token identifier for the up-axis command. */
    private static final int UP_AXIS = 1;

    /** Token identifier for the rotation command. */
    private static final int ROTATION = 2;

    /** Token identifier for the zoom command. */
    private static final int ZOOM = 3;

    /** Token identifier for the invkin command. */
    private static final int INVKIN = 4;

    /** Token identifier for the src command. */
    private static final int SRC = 5;

    /** Token identifier for the modelparser command. */
    private static final int MODELPARSER = 6;

    /** Token identifier for the link length command. */
    private static final int LENGTH = 7;

    /** Token identifier for the degree of freedom command.*/
    private static final int DOF = 8;

    /** Token identifier for the name command. */
    private static final int NAME = 9;

    /** Token identifier for the type of degree of freedom command. */
    private static final int TYPE = 10;

    /** Token identifier for the revolute joint type. */
    private static final int TYPE_REVOLUTE = 11;

    /** Token identifier for the prismatic joint type. */
    private static final int TYPE_PRISMATIC = 12;

    /** Token identifier for the inanimate joint type. */
    private static final int TYPE_INANIMATE = 13;

    /** Token identifier for the joint offset command. */
    private static final int OFFSET = 14;

    /** Token identifier for the joint twist command. */
    private static final int TWIST = 15;

    /** Token identifier for the minimum joint variable command. */
    private static final int MIN = 16;

    /** Token identifier for the maximum joint variable command. */
    private static final int MAX = 17;

    /** Token identifier for the home join variable command. */
    private static final int HOME = 18;

    /** Token for the end of the degree of freedom command. */
    private static final int DOF_END = 19;

    /** Token for the end of the joint definition command. */
    private static final int JOINT_END = 20;

    /** Token for the end of definition command. */
    private static final int END = 21;

    /** The Hashtable of language resource. */
    private Hashtable theResourceTable;

    /** The Robot object, which holds data parsed by this object. */
    private Robot theRobot = Robot.getInstance();

    /** Default language resource. */
    private String[][] resources = 
    {
        {"syntax.upaxis", "up-axis"},
        {"syntax.rotation", "rotation"},
        {"syntax.zoom", "zoom"},
        {"syntax.invkin", "invkin"},
        {"syntax.src", "src"},
        {"syntax.modelparser", "modelparser"},
        {"syntax.length", "length"},
        {"syntax.dof", "dof"},
        {"syntax.name", "name"},
        {"syntax.type", "type"},
        {"syntax.type.revolute", "revolute"},
        {"syntax.type.prismatic", "prismatic"},
        {"syntax.type.inanimate", "inanimate"},
        {"syntax.offset", "offset"},
        {"syntax.twist", "twist"},
        {"syntax.min", "min"},
        {"syntax.max", "max"},
        {"syntax.home", "home"},
        {"syntax.dofend", "dof-end"},
        {"syntax.jointend", "joint-end"},
        {"syntax.end", "end"},
        {"errors.syntax.upaxis", "Invalid UP-AXIS specifier. UP-AXIS must be followed by either X, Y, or Z."},
        {"errors.syntax.rotation", "Invalid ROTATION specifier. ROTATION must followed by an angle in degrees."},
        {"errors.syntax.zoom", "Invalid ZOOM specifier.  ZOOM must be followed by a number defining the initial view zoom factor."},
        {"errors.syntax.invkin", "Invalid INVKIN specifier. INVKIN must be followed by a String specifying the inverse kinematics class for the simulation."},
        {"errors.syntax.src", "Invalid SRC specifier. SRC must be followed by a String specifying the file for the 3D model."},
        {"errors.syntax.modelparser", "Invalid MODELPARSER specifier. MODELPARSER must be followed by a String specifying the parser for the model."},
        {"errors.syntax.length", "Invalid LENGTH specifier. LENGTH must be followed by a number specifying the link length."},
        {"errors.syntax.degrees", "Invalid DOF specifier. DOF must be followed by a number specifying the number of degrees of freedom in the joint."},
        {"errors.syntax.name", "Invalid NAME specifier. NAME must be followed by a String specifying the degree name."},
        {"errors.syntax.type", "Invalid TYPE specifier. TYPE must be followed by a String specifying the degree type as either REVOLUTE or PRISMATIC."},
        {"errors.syntax.offset", "Invalid OFFSET specifier. OFFSET must be followed by a number specifying the link offset."},
        {"errors.syntax.twist", "Invalid TWIST specifier. TWIST must be followed by a number specifying the link twist."},
        {"errors.syntax.min", "Invalid MIN specifier. MAX must be followed by a number specifying the minimum link variable value."},
        {"errors.syntax.max", "Invalid MAX specifier. MAX must be followed by a number specifying the maximum link variable value."},
        {"errors.syntax.home", "Invalid HOME specifier. HOME must be followed by a number specifying the link variable home value."}
    };


    /**
	 * The default constructor which retrieves the language properties, fills
	 * the resource table, and initializes the class variables.
	 */
    protected DefaultRobotParser()
    {
        theResourceTable = new Hashtable();

        fillResources();
    }

    /**
	 * Fills up the resources using the hardcoded and soft-loaded resources.
	 */
    public void fillResources()
    {
        try
        {
            ResourceBundle bundle = ResourceBundle.getBundle("RobotParserConfig");
            theResourceTable = new Hashtable();

            for (int i = 0; i < resources.length; i++)
            {
                String temp;

                try
                {
                    temp = bundle.getString(resources[i][0]);
                }
                catch (MissingResourceException mre)
                {
                    temp = resources[i][1];
                }

                theResourceTable.put(resources[i][0], temp);
            }
        }
        catch (MissingResourceException mre)
        {
            System.err.println("DefaultRobotParser: Unable to find language resources.");
        }
    }

    /**
	 * Reads in the robot configuration from the Reader stream.
	 *
	 * @param reader the configuration reader name.
	 * @exception RobotParserException indicates an error parsing the 
     *            configuration file.
	 */
    public void read(Reader reader) throws RobotParserException
    {
        StreamTokenizer st = new StreamTokenizer(reader);
        Hashtable theNameTable = new Hashtable();
        JointVariables theJointVariables = new JointVariables();
        boolean doContinue = true;
        Joint currentJoint = new Joint();
        Degree currentDegree = new Degree();
        int jointCounter = 0;
        int degreeCounter = 0;

        st.slashSlashComments(true);
        st.slashStarComments(true);

        try
        {
            while (doContinue == true)
            {
                switch (st.nextToken())
                {
                    case StreamTokenizer.TT_WORD:
                    {
                        switch (decode(st.sval))
                        {
                            case UP_AXIS:
                            {
								if (st.nextToken() == StreamTokenizer.TT_WORD)
                                {
                                    if (st.sval.equalsIgnoreCase("X"))
                                    {
                                        theRobot.setUpAxis(Robot.X_UP);
                                    }
                                    else if (st.sval.equalsIgnoreCase("Y"))
                                    {
                                        theRobot.setUpAxis(Robot.Y_UP);
                                    }
                                    else if (st.sval.equalsIgnoreCase("Z"))
                                    {
                                        theRobot.setUpAxis(Robot.Z_UP);
                                    }
                                    else
                                    {
                                        throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.upaxis"));
                                    }
                                }
                                else
                                {
                                    throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.upaxis"));
                                }

                                break;
                            }

                            case ROTATION:
                            {
                                if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                {
                                    theRobot.setRotation((double)st.nval);
                                }
                                else
                                {
                                    throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.rotation"));
                                }

                                break;
                            }

                            case ZOOM:
                            {
                                if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                {
                                    theRobot.setZoom((double)st.nval);
                                }
                                else
                                {
                                    throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.zoom"));
                                }

                                    break;
                            }

                            case INVKIN:
                            {
                                if (st.nextToken() == StreamTokenizer.TT_WORD)
                                {
                                    try
                                    {
                                        theRobot.setInvKin(InvKin.loadKinematics(st.sval));
                                    }
                                    catch (InvKinException ike)
                                    {
                                        throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.invkin"));
                                    }
                                }
                                else
                                {
                                    throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.invkin"));
                                }

                                break;
                            }

                            case SRC:
                            {
                                if (st.nextToken() == StreamTokenizer.TT_WORD)
                                {
                                    currentJoint.setSource(st.sval);
                                    jointCounter++;
                                    degreeCounter =  0;
                                }
                                else
                                {
                                    throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.src"));
                                }

                                break;
                            }

                            case MODELPARSER:
                            {
                                if (st.nextToken() == StreamTokenizer.TT_WORD)
                                {
                                    try
                                    {
                                        currentJoint.setModelParser(st.sval);
                                    }
                                    catch (ModelParserException mpe)
                                    {
                                        throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.modelparser"));
                                    }
                                }
                                else
                                {
									throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.modelparser"));
                                }

                                break;
                            }

                            case LENGTH:
                            {
                                if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                {
                                    currentJoint.setLength(st.nval);
                                }
                                else
                                {
                                    throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.length"));
                                }

                                break;
                            }

                            case DOF:
                            {
                                int degrees =  1;

                                if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                {
                                    degrees = (int) st.nval;
                                }
                                else
                                {
                                    throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.degrees"));
                                }

                                for (int i = 1; i <= degrees; i++)
                                {
                                    boolean done = false;
                                    currentDegree = new Degree();

                                    while (done == false)
                                    {
                                        switch (st.nextToken())
                                        {
                                            case StreamTokenizer.TT_WORD:
                                            {
                                                switch (decode(st.sval))
                                                {
                                                    case NAME:
                                                    {
                                                        if (st.nextToken() == StreamTokenizer.TT_WORD)
                                                        {
                                                            currentDegree.setName(st.sval);
                                                            degreeCounter++;
                                                            theNameTable.put(st.sval.toUpperCase(), new IndexPair(jointCounter, degreeCounter));
                                                        }
                                                        else
                                                        {
                                                            throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.name"));
                                                        }

                                                        break;
                                                    }

                                                    case TYPE:
                                                    {
                                                        if (st.nextToken() == StreamTokenizer.TT_WORD)
                                                        {
                                                            switch (decode(st.sval))
                                                            {
                                                                case TYPE_REVOLUTE:
                                                                {
                                                                    currentDegree.setType(Degree.TYPE_REVOLUTE);

                                                                    break;
                                                                }

                                                                case TYPE_PRISMATIC:
                                                                {
                                                                    currentDegree.setType(Degree.TYPE_PRISMATIC);

                                                                    break;
                                                                }

                                                                case TYPE_INANIMATE:
                                                                {
                                                                    currentDegree.setType(Degree.TYPE_INANIMATE);

                                                                    break;
                                                                }

                                                                default:
                                                                {
                                                                    throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.type"));
                                                                }
                                                            }
                                                        }

                                                        break;
                                                    }

                                                    case OFFSET:
                                                    {
                                                        if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                                        {
                                                            currentDegree.setOffset(st.nval);
                                                        }
                                                        else
                                                        {
                                                            throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.offset"));
                                                        }

                                                        break;
                                                    }

                                                    case TWIST:
                                                    {
                                                        if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                                        {
                                                            currentDegree.setTwist(st.nval);
                                                        }
                                                        else
                                                        {
                                                            throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.twist"));
                                                        }

                                                        break;
                                                    }

                                                    case MIN:
                                                    {
                                                        if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                                        {
                                                            currentDegree.setMinimum(st.nval);
                                                        }
                                                        else
                                                        {
                                                            throw  new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.min"));
                                                        }

                                                        break;
                                                    }

                                                    case MAX:
                                                    {
                                                        if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                                        {
                                                             currentDegree.setMaximum(st.nval);
                                                        }
                                                        else
                                                        {
                                                             throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.max"));
                                                        }

                                                        break;
                                                    }

                                                    case HOME:
                                                    {
                                                        if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                                                        {
                                                            currentDegree.setHome(st.nval);
                                                            currentDegree.setVariable(st.nval);
                                                        }
                                                        else
                                                        {
                                                            throw new RobotParserException(st.lineno() + ": " + (String)theResourceTable.get("errors.syntax.home"));
                                                        }

                                                        break;
                                                    }

                                                    case DOF_END:
                                                    {
                                                        currentJoint.addDegree(currentDegree);
                                                        currentDegree = new Degree();
                                                        done = true;

                                                        break;
                                                    }

                                                    case StreamTokenizer.TT_EOF:
                                                    {
                                                        throw new RobotParserException("");
                                                    }
                                                }

                                                break;
                                            }

                                            case StreamTokenizer.TT_EOF:
                                            {
												doContinue = false;
                                                break;
                                            }
                                        }
                                    }
                                }

                                break;
                            }

                            case JOINT_END:
                            {
                                theJointVariables.addJoint(currentJoint);
                                currentJoint = new Joint();

                                break;
                            }

                            case END:
                            {
                                doContinue = false;

                                break;
                            }

                            case NULL:
                            {
                                doContinue = false;

                                break;
                            }

                            default:
                            {
                                doContinue = false;

                                break;
                            }
                        }

                        break;
                    }

                    case StreamTokenizer.TT_EOF:
                    {
                        doContinue = false;

                        break;
                    }
                }
            } 
        
            theJointVariables.setNameTable(theNameTable);
        }
        catch (IOException ie)
        {
            throw new RobotParserException(ie.getMessage());
        }
        finally
        {
            try
            {
                reader.close();
            }
            catch (IOException ie)
            {
                throw new RobotParserException(ie.getMessage());
            }
        }

        theRobot.setJointVariables(theJointVariables);
    }

    /**
	 * Decodes the given token and returns a constant.
	 *
	 * @param s theString to decode tokens.
	 * @return a constant.
	 */
    public int decode(String s)
    {
        if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.upaxis")))
        {
            return UP_AXIS;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.rotation")))
        {
            return ROTATION;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.zoom")))
        {
            return ZOOM;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.invkin")))
        {
            return INVKIN;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.src")))
        {
            return SRC;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.modelparser")))
        {
            return MODELPARSER;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.length")))
        {
            return LENGTH;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.dof")))
        {
            return DOF;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.name")))
        {
            return NAME;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.type")))
        {
            return TYPE;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.type.revolute")))
        {
            return TYPE_REVOLUTE;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.type.prismatic")))
        {
            return TYPE_PRISMATIC;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.type.inanimate")))
        {
            return TYPE_INANIMATE;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.offset")))
        {
            return OFFSET;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.twist")))
        {
            return TWIST;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.min")))
        {
            return MIN;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.max")))
        {
            return MAX;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.home")))
        {
            return HOME;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.dofend")))
        {
            return DOF_END;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.jointend")))
        {
            return JOINT_END;
        }
        else if (s.equalsIgnoreCase((String) theResourceTable.get("syntax.end")))
        {
            return END;
        }
        else
        {
            return NULL;
        }
    }

    /**
	 * Returns the Robot representation of the robot to be simulated. 
	 *
	 * @param filename the name of the configuration file.
	 * @return the representation of the robot.
	 */
    public Robot getRobot()
    {
        return theRobot;
    }
}