/*
 * DefineNamedXYZCommand.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import javax.vecmath.Point3d;


/**
 * A class that encapsulates a command to define a named endpoint and save it 
 * in the simulation, for use at a later time in the simulation. This class is
 * a concrete implementation of the Command class, and is a part of REMOTE's 
 * implementation of the Command Pattern, as demonstrated in Gamma et al (page 
 * 236).
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 * @see SimulationEngine
 * @see CommandQueue
 */
public class DefineNamedXYZCommand extends Command
{
    /**
     * The name for the endpoint.
     */
    private String name;

    /**
     * The x-coordinate of the endpoint.
     */
    private double x;

    /**
     * The y-coordinate of the endpoint.
     */
    private double y;

    /**
     * The z-coordinate of the endpoint.
     */
    private double z;


    /**
     * Creates a new DefineNamedXYZCommand, which will associate a name with an
     * endpoint in the simulation.
     * 
     * @param lineNumber the source line for this command.
     * @param name the name for the endpoint.
     * @param x the x-coordinate of the endpoint.
     * @param y the y-coordinate of the endpoint.
     * @param z the z-coordinate of the endpoint.
     */
    public DefineNamedXYZCommand(int lineNumber, String name, double x, double y, double z)
    {
        super(lineNumber);
        
        // Save the name.
        this.name = name;

        // Save the endpoint coordinates.
        this.x = x;
        this.y = y;
        this.z = z;
    }

    /**
     * Associates a name with the given endpoint coordinates, using the 
     * XYZTable singleton instance.
     *
     * @exception thrown if an error occurs attempting to execute the command.
     */
    public void Execute() throws Exception
    {
        // Get the table instance.
        XYZTable theXYZTable = XYZTable.getInstance();

        // Save the named coordinate.
        theXYZTable.putNamedEndpoint(name, new Point3d(x, y, z));
    }
}