/*
 * Degree.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import  javax.media.j3d.*;

/**
 * A class to encapsulate a degree of freedom in the Robot.  This class has
 * member fields which describe most of the Denavit-Hartenberg parameters
 * required to represent the transformations from one degree of freedom to 
 * the next.<p>
 *
 * A Degree can be one of three types: revolute, prismatic, or inanimate. An
 * inanimate degree of freedom is used merely as a placeholder in the Robot, 
 * and cannot be moved via Commands.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 * @see Robot
 * @see Joint
 */
public class Degree implements Cloneable
{
    /** Specifies a revolute-type degree of freedom. */
    public final static int TYPE_REVOLUTE =  0;

    /** Specifies a prismatic-type degree of freedom. */
    public final static int TYPE_PRISMATIC =  1;

    /** Specifies an inanimte degree of freedom, used as a placeholder. */
    public final static int TYPE_INANIMATE =  2;

    /** The name of this Degree. */
    private String name;

    /** The type of the degree of freedom's motion. */
    private int type;

    /** The link twist, given in degrees. */
    private double twist;

    /** The link offset, along the Z-axis. */
    private double offset;

    /** The minimum degree variable value. */
    private double minimum;

    /** The maximum degree variable value. */
    private double maximum;

    /** The home degree variable value. */
    private double home;

    /** The degree variable */
    private double variable;

    /** Indicates if any of the member fields have been initialized. */
    private boolean isEmpty;

    /** Performs the transformation from the previous degree to the next. */
    private TransformGroup transformGroup;


    /**
     * Creates and initializes a new degree of freedom.
     */
    public Degree()
    {
        type = TYPE_INANIMATE;
        twist = 0.0;
        offset = 0.0;
        minimum = 0.0;
        maximum = 0.0;
        home = 0.0;
        variable = 0.0;
        isEmpty = true;
        transformGroup = new TransformGroup();
    }

    /**
     * Sets the Degree's name.
     *
     * @param name the name for the Degree.
     */
    public void setName(String name)
    {
        this.name = name;
        isEmpty = false;
    }

    /**
     * Gets the name of Degree.
     *
     * @return the name of the Degree.
     */
    public String getName()
    {
        return name;
    }

    /**
     * Sets the type of Degree: TYPE_REVOLUTE, TYPE_PRISMATIC, or TYPE_INANIMATE.
     *
     * @param type the integer representation of the type of Degree.
     */
    public void setType(int type)
    {
        this.type = type;
        isEmpty = false;
    }

    /**
     * Gets the type of Degree: TYPE_REVOLUTE, TYPE_PRISMATIC, or TYPE_INANIMATE.
     *
     * @return an integer representation of the type of Degree.
     */
    public int getType()
    {
        return type;
    }

    /**
     * Sets the Degree's offset.
     *
     * @param offset the offset for the Degree.
     */
    public void setOffset(double offset)
    {
        this.offset = offset;
        isEmpty = false;
    }

    /**
     * Gets the Degree's offset.
     *
     * @return the offset for the Degree.
     */
    public double getOffset()
    {
        return offset;
    }

    /**
     * Sets the Degree's twist.
     *
     * @param the setting for the Degree twist (in degrees).
     */
    public void setTwist(double twist)
    {
        this.twist = twist;
        isEmpty = false;
    }

    /**
     * Gets the Degree's twist.
     *
     * @return the Degree's twist (in degrees).
     */
    public double getTwist()
    {
        return twist;
    }

    /**
     * Sets the Degree's minimum variable value.
     *
     * @param minimum the minimum setting for the Degree variable.
     */
    public void setMinimum(double minimum)
    {
        this.minimum = minimum;
        isEmpty = false;
    }

    /**
     * Gets the Degree's minimum variable value.
     *
     * @return the Degree's minimum variable value.
     */
    public double getMinimum()
    {
        return minimum;
    }

    /**
     * Sets the Degree's maximum variable value.
     *
     * @param maximum the maximum setting for the Degree variable.
     */
    public void setMaximum(double maximum)
    {
        this.maximum = maximum;
        isEmpty = false;
    }

    /**
     * Gets the Degree's maximum variable value.
     *
     * @return the Degree's maximum variable value.
     */
    public double getMaximum()
    {
        return maximum;
    }

    /**
     * Sets the Degree's home variable value.
     *
     * @param home the home setting for the Degree variable.
     */
    public void setHome(double home)
    {
        this.home = home;
        isEmpty = false;
    }

    /**
     * Get the Degree's home variable value.
     *
     * @return double the Degree's variable home value.
     */
    public double getHome()
    {
        return home;
    }

    /**
     * Sets the Degree's variable
     *
     * @param variable the value for the Degree variable.
     */
    public void setVariable(double variable)
    {
        this.variable = variable;
        isEmpty = false;
    }

    /**
     * Gets the Degree's variable.
     *
     * @return the Degree's variable value.
     */
    public double getVariable()
    {
        return variable;
    }

    /**
     * Sets the TransformGroup required to transform from one degree to the next.
     *
     * @param transformGroup performs the Degree's transformation.
     */
    public void setTransformGroup(TransformGroup transformGroup)
    {
        this.transformGroup = transformGroup;
    }

    /**
     * Returns the TransformGroup performing the Degree's transformation.
     *
     * @return the TransformGroup performing the transformation.
     */
    public TransformGroup getTransformGroup()
    {
        return transformGroup;
    }

    /**
     * Determines if the given variable is within the Degree's range of motion.
     *
     * @param variable the value to be range-tested.
     * @return a boolean indicating if the variable is in the Degree's range.
     */
    public boolean inRange(double variable)
    {
        if ((variable >= getMinimum()) && (variable <= getMaximum()))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Determines if the Degree's current variable is within variable range.
     *
     * @return indicates if the current variable is within range of motion.
     */
    public boolean inRange()
    {
        return inRange(getVariable());
    }

    /**
     * Returns if the Degree is empty.
     *
     * @return indicates if the Degree object is empty.
     */
    public boolean isEmpty()
    {
        return isEmpty;
    }

    /**
     * Implements the clone() method, returns an Object copy of the Degree.
     *
     * @return the Object copy of this object.
     */
    public Object clone()
    {
        Degree newDegree = new Degree();
        newDegree.name =  name;
        newDegree.type =  type;
        newDegree.twist = twist;
        newDegree.offset = offset;
        newDegree.minimum = minimum;
        newDegree.maximum = maximum;
        newDegree.home = home;
        newDegree.variable = variable;
        newDegree.isEmpty = isEmpty;
        Transform3D t3d =  new Transform3D();
        transformGroup.getTransform(t3d);
        newDegree.transformGroup.setTransform(t3d);

        return  (Object) newDegree;
    }
}