/*
 * GotoNamedXYZCommand.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import javax.vecmath.Point3d;


/**
 * A class that encapsulates a command to move the robot to a defined endpoint.
 * This class is a concrete implementation of the Command class, and is a part 
 * of REMOTE's implementation of the Command Pattern, as demonstrated in Gamma 
 * et al (page 236).
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 * @see SimulationEngine
 * @see CommandQueue
 */
public class GotoNamedXYZCommand extends Command
{
    /**
     * The name of the defined endpoint.
     */
    private String name;


    /**
     * Creates a new GotoNamedXYZCommand, which will move the Robot to the 
     * previously defined endpoint associated with the given name.
     *
     * @param lineNumber the source line for this command.
     * @param name the name of the endpoint definition.
     */
    public GotoNamedXYZCommand(int lineNumber, String name)
    {
        super(lineNumber);

        // Save the name.
        this.name = name;
    }

    /**
     * Moves the Robot to a previously defined endpoint, as saved in the 
     * XYZTable singleton instance.
     *
     * @exception thrown if an error occurs attempting to execute the command.
     */
    public void Execute() throws Exception
    {
        // Get the table instance.
        XYZTable theXYZTable = XYZTable.getInstance();

        // Retrieve the named coordinate.
        Point3d endpoint = theXYZTable.getNamedEndpoint(name);

        // Get the Robot instance.
        Robot theRobot = Robot.getInstance();

        // Move the robot to the endpoint coordinates.
        theRobot.moveToXYZ(endpoint.x, endpoint.y, endpoint.z);
    }
}