/*
 * HelpFrame.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.IOException;

/**
 * A Frame to give help information from HTML files.
 *
 * @author Brendon Wilson
 * @version alpha
 */
public class HelpFrame extends Frame implements HyperlinkListener, WindowListener
{
	JEditorPane html;
	JPanel panel = new JPanel();
	
	
	/**
	 * Public constructor, sets up the help dialog using the 
	 * provided document path.
	 *
	 * @param parent the parent Frame.
	 * @param title the title for the HelpDialog.
	 * @param path the path to the help html-documents.
	 * @exception MalformedURLException if the path string is not a valid URL.
	 * @exception IOException if there is a problem reading the specfied URL.
	 */
	public HelpFrame(String title, String path) throws MalformedURLException, IOException
	{
		super(title);
		setIconImage((new ImageIcon("images" + System.getProperty("file.separator") + "remote.gif")).getImage());
		
		panel.setBorder(new EmptyBorder(5,5,5,5));
		panel.setLayout(new BorderLayout());
		
		URL url = new URL(path);
		html = new JEditorPane(url);
		html.setEditable(false);
		html.addHyperlinkListener(this);
		
		JScrollPane scroller = new JScrollPane();
		scroller.setBorder(new BevelBorder(BevelBorder.LOWERED, Color.white, Color.gray));
		
		JViewport vp = scroller.getViewport();
		vp.add(html);
		vp.setBackingStoreEnabled(true);
		
		panel.add(scroller, BorderLayout.CENTER);
		
		this.add(panel);
		this.pack();
		
		Toolkit theToolkit = Toolkit.getDefaultToolkit();
		Dimension theDimension = theToolkit.getScreenSize();
		this.setSize((int) (theDimension.width * 0.50), (int) (theDimension.height * 0.60));
		int x = theDimension.width / 2 - this.getSize().width / 2;
		int y = theDimension.height / 2 - this.getSize().height / 2;
		
		this.setLocation(x, y);
		this.addWindowListener(this);
	}
	
	/**
	 * Set location for the help document.
	 *
	 * @param location the String of the document path.
	 * @exception MalformedURLException if the location string is not a valid URL.
	 */
	public void setLocation(String location) throws MalformedURLException
	{
		linkActivated(new URL(location));
	}
	
	
	/**
	 * Notification of a change relative to a hyperlink.
	 *
	 * @param e the HypelinkEvent to be handled.
	 */
	public void hyperlinkUpdate(HyperlinkEvent e)
	{
		if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
		{
	    		linkActivated(e.getURL());
		}
	}
	
	/**
	 * Follows the reference in an
	 * link.  The given url is the requested reference.
	 * By default this calls <a href="#setPage">setPage</a>,
	 * and if an exception is thrown the original previous
	 * document is restored and a beep sounded.  If an 
	 * attempt was made to follow a link, but it represented
	 * a malformed url, this method will be called with a
	 * null argument.
	 *
	 * @param u the URL to follow.
	 */
	protected void linkActivated(URL u)
	{
		Cursor c = html.getCursor();
		Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
		html.setCursor(waitCursor);
		SwingUtilities.invokeLater(new PageLoader(u, c));
	}
	
	/**
	 * Deals with closing the Dialog when user request window to close.
	 *
	 * @param e the WindowEvent to be handled.
	 */
	public void windowClosing(WindowEvent e)
	{
		this.setVisible(false);
		this.dispose();	
	}
	
	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}
	
	/**
	 * Class that loads page synchronously.
	 */
	class PageLoader implements Runnable
	{
		URL url;
		Cursor cursor;
		
		PageLoader(URL u, Cursor c)
		{
			url = u;
			cursor = c;
		}
		
		public void run()
		{
			if (url == null)
			{
				// restore the original cursor
				html.setCursor(cursor);
				
				// PENDING(prinz) remove this hack when 
				// automatic validation is activated.
				Container parent = html.getParent();
				html.invalidate();
				html.validate();
				parent.repaint();
			}
			else
			{
				Document doc = html.getDocument();
				try
				{
					html.setPage(url);
				}
				catch (IOException ioe)
				{
					html.setDocument(doc);
					getToolkit().beep();
				}
				finally
				{
					// schedule the cursor to revert after
					// the paint has happended.
					url = null;
					SwingUtilities.invokeLater(this);
				}
			}
		}
	}
}
