/*
 * InvKin.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import javax.vecmath.*;

/**
 * An abstract class to encapsulate the functionality required to perform the 
 * inverse kinematics calculations required by the Robot object to execute 
 * Command.MOVE_XYZ or Command.GOTO_XYZ commands.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 *
 * @see JointVariables
 * @see Command
 */
public abstract class InvKin implements Cloneable
{
    /** Conversion constant, to convert from radians to degrees. */
    protected final double TO_DEGREES =  360/(2*Math.PI);

    /** Conversion constant, to convert from degrees to radians. */
    protected final double TO_RADIANS =  (2*Math.PI)/360;

    /** The set of Joints in the Robot. */
    private JointVariables joints;


    /**
     * Creates a new InvKin object.  This constructor is private to prevent 
     * direct instantiation; use <code>loadParser</code> to load an instance 
     * of a specific inverse kinematic object.
     */
    protected InvKin()
    {
        super();
    }

    /**
     * Sets the JointVariables object.
     *
     * @param joints the new JointVariables object.
     */
    public void setJointVariables(JointVariables joints)
    {
        this.joints = joints;
    }

    /**
     * Get the JointVariables object.
     *
     * @return the current JointVariables object.
     */
    public JointVariables getJointVariables()
    {
        return joints;
    }

    /**
     * Returns the closest solution in JointVariables form, if a solution 
     * exists, otherwise returns null. Modified from the equations given in 
     * Craig, "Introduction to Robotics".  Note that the desired pitch angle 
     * is implicitly equal to the current pitch, and the desired roll angle is 
     * the current roll.
     *
     * @param target the target Point3d to use to find the solution.
     * @returns the closest solution.
     * @exception OutOfRangeException raised when the requested point has no solutions.
     */
    public abstract JointVariables getJointVariables(Point3d target)
        throws OutOfRangeException;

    /**
     * Loads the InvKin class specified by the given String.
     *
     * @param s the name of the InvKin class to be loaded.
     * @return an instance of the requested InvKin class.
     * @exception InvKinException indicates an error loading the class.
     */
    public static InvKin loadKinematics(String s)
        throws InvKinException
    {
        Class aClass;
        InvKin aInvKin;

        try
        {
            aClass = Class.forName(s);
            aInvKin = (InvKin) aClass.newInstance();

            return aInvKin;
        }
        catch (Exception e)
        {
            throw new InvKinException(e.getMessage());
        }
    }

    /**
     * Implements the clone() method, returning an Object copy of this object.
     
     * @return an Object copy of this InvKin instance.
     */
    public abstract Object clone ();
}