/*
 * Joint.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import  java.io.*;
import  java.util.*;
import  javax.media.j3d.*;

/**
 * A class that encapsulates a robot Joint, including the 3D representation  
 * that will be used to render the Joint, a Degree object for each of the 
 * Joint's degrees of freedom, and the Joint's link length.  All of the 
 * other Denavit-Hartenberg parameters are included in each Degree object.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 *
 * @see Robot
 * @see Degree
 */
public class Joint implements Cloneable
{
    /** The name of the 3D model file. */
    private String source;

    /** The link length of this Joint. */
    private double length;

    /** The Hashtable of Degree (of freedom) objects. */
    private Hashtable degrees;

    /** The key used for putting Degree objects into the Hashtable. */
    private int key;

    /** The 3D representation of the Joint. */
    private Group model;


    /**
     * Creates and initializes a new joint.
     */
    public Joint()
    {
        degrees = new Hashtable();
        length = 0.0;
        key = 1;
    }

    /**
     * Add a Degree to the Joint.
     *
     * @param degree the Degree object to be added to the Joint.
     */
    public void addDegree(Degree degree)
    {
        degrees.put(new Integer(key), degree);
        key++;
    }

    /**
     * Returns the specified Degree.
     *
     * @param i the index of the requested Degree.
     * @return the requested Degree.
     */
    public Degree getDegree(int i)
    {
        return (Degree) degrees.get(new Integer(i));
    }

    /**
     * Returns an Enumeration all of the Degrees.
     *
     * @return the Enumeration of all Degrees in the Joint.
     */
    public Enumeration getDegrees()
    {
        return degrees.elements();
    }

    /**
     * Returns the number of degrees of freedom in this Joint.
     *
     * @return the number of degrees of freedom in the Joint.
     */
    public int countDegrees()
    {
        return degrees.size();
    }

    /**
     * Returns true if the Joint is empty.
     *
     * @return true if the Joint is empty.
     */
    public boolean isEmpty()
    {
        return degrees.isEmpty();
    }

    /**
     * Sets the source filename of the joint's model representation.
     *
     * @param source the name of the model source file.
     */
    public void setSource(String source)
    {
        this.source = source;
    }

    /**
     * Returns the 3D representation of this Joint.
     *
     * @return the 3D representation of the Joint.
     */
    public Group getModel()
    {
        return model;
    }

    /**
     * Set the ModelParser to be used to interpret the source file. This 
     * method must be called after the source filename has been set, otherwise
     * a ModelParserException will occur when the ModelParser class attempts to
     * load the source file.
     *
     * @param modelParser the name of the ModelParser class to parse the source.
     * @exception ModelParserException indicates an error loading the 3D model.
     */
    public void setModelParser(String modelParser) throws ModelParserException
    {
        ModelParser aParser = ModelParser.loadParser(modelParser);
        aParser.read(source);

        this.model = aParser.getModel();
    }

    /**
     * Sets the link length of the Joint.
     *
     * @param length the length of the link.
     */
    public void setLength(double length)
    {
        this.length = length;
    }

    /**
     *
     * Returns the link length of the Joint.
     *
     * @return the length of the link.
     */
    public double getLength()
    {
        return length;
    }

    /**
     * Implements the clone() method, returning an Object copy of the Joint.
     *
     * @return the Object copy of the Joint.
     */
    public Object clone()
    {
        Joint newJoint = new Joint();
        newJoint.source = source;
        newJoint.length = length;
        newJoint.model = model;

        // Hack to work around shallow nature of Hashtable.clone() method.
        for (int i =  1; i <= degrees.size(); i++)
        {
            Degree newDegree = (Degree) getDegree(i).clone();
            newJoint.addDegree(newDegree);
        }

        return (Object) newJoint;
    }
}