/*
 * JointVariables.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import  java.util.*;

/**
 * A class to hold the set of Joint objects for the Robot.  Although traditional
 * robotics literature uses the term 'joint variables' to describe the position
 * of specific joint, this class uses the same term because in a round about 
 * way it does the same thing: specifying the position of the Robot's degrees
 * of freedom.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 *
 * @see SimulationEngine
 * @see Robot
 * @see Joint
 * @see Degree
 */
public class JointVariables implements Cloneable
{
    /** Hashtable associating a names with Degrees, using IndexPair objects. */
    private Hashtable names;

    /** A Hashtable of Joint objects. */
    private Hashtable joints;

    /** The key used to put Joints into the <pre>joints</pre> Hashtable. */
    private int key;


    /**
     * Class constructor, initializes the variables.
     */
    public JointVariables()
    {
        names = new Hashtable();
        joints = new Hashtable();
        key = 1;
    }

    /**
     * Add a joint to the set of joint variables.
     *
     * @param joint the Joint to be added to the JointVariables object.
     */
    public void addJoint(Joint joint)
    {
        joints.put(new Integer(key++), joint);
    }

    /**
     * Returns the requested joint.
     *
     * @param index the index of the requested joint.
     * @return the requested Joint.
     */
    public Joint getJoint(int index)
    {
        return (Joint) joints.get(new Integer(index));
    }

    /**
     * Returns the Joint containing the named Degree.
     *
     * @param name the name of the Joint to return.
     * @return the Joint containing the named Degree.
     */
    public Joint getJoint(String name)
    {
        IndexPair param = (IndexPair) names.get(name);
        int joint = param.joint;

        return (Joint) joints.get(new Integer(joint));
    }

    /**
     * Returns the named Degree.
     *
     * @param name the name of the requested Degree.
     * @return the requested Degree.
     */
    public Degree getDegree(String name)
    {
        IndexPair param = (IndexPair) names.get(name);
        int joint = param.joint;
        int degree = param.degree;
        
        Joint aJoint = (Joint) joints.get(new Integer(joint));

        return (Degree) aJoint.getDegree(degree);
    }

    /**
     * Returns all the Joint objects as an Enumeration object.
     *
     * @return the Enumeration of Joint objects.
     */
    public Enumeration getJoints()
    {
        return joints.elements();
    }

    /**
     * Returns the number of Joints in the JointVariables object.
     *
     * @return the number of Joints in the object.
     */
    public int countJoints()
    {
        return joints.size();
    }

    /**
     * Sets the Hashtable associating names with an IndexPair object, allowing
     * a specific Degree to be fetched by name.
     *
     * @param names the Hashtable of names.
     */
    public void setNameTable(Hashtable names)
    {
        this.names = names;
    }

    /**
     * Returns the current Hashtable associating names with IndexPair objects.
     *
     * @return the current name Hashtable being used.
     */
    public Hashtable getNameTable()
    {
        return names;
    }

    /**
     * Determines if the set of joint angles is within the joint space.
     *
     * @return indicates true if the JointVariables object is inside joint space.
     */
    public boolean IsInRange()
    {
        boolean inRange = true;
        Enumeration JointEnum = joints.elements();

        while ((inRange == true) && JointEnum.hasMoreElements())
        {
            Enumeration DegreeEnum = ((Joint) JointEnum.nextElement()).getDegrees();

            while ((inRange == true) && DegreeEnum.hasMoreElements())
            {
                Degree currentDegree = (Degree) DegreeEnum.nextElement();

                if (!currentDegree.inRange())
                {
                    inRange = false;
                }
            }
        }

        return inRange;
    }

    /**
     * Returns the Robot to its home position, as specified by the 
     * configuration parameters used to create the robot.
     */
    public void home()
    {
        Enumeration theJointsEnum = getJoints();
        Joint currentJoint;

        while (theJointsEnum.hasMoreElements())
        {
            currentJoint = (Joint)theJointsEnum.nextElement();
            Enumeration theDegreesEnum = currentJoint.getDegrees();
            Degree currentDegree;

            while (theDegreesEnum.hasMoreElements())
            {
                currentDegree = (Degree)theDegreesEnum.nextElement();
                currentDegree.setVariable(currentDegree.getHome());
            }
        }
    }

    /**
     * Implements clone(), returning an Object copy of the JointVariables.
     *
     * @return the Object copy of this object.
     */
    public Object clone()
    {
        JointVariables newJointVariables = new JointVariables();

        for (int i = 1; i <= joints.size(); i++)
        {
            newJointVariables.addJoint((Joint) getJoint(i).clone());
        }

        // Hack to get around shallow nature of Hashtable.clone() method.
        Hashtable newNameTable = new Hashtable();
        Enumeration TableEnum = names.keys();

        while (TableEnum.hasMoreElements())
        {
            String key = (String) TableEnum.nextElement();
            newNameTable.put(key, names.get(key));
        }

        newJointVariables.setNameTable(newNameTable);
        return (Object) newJointVariables;
    }
}