/*
 * PumaInvKin.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import javax.vecmath.*;

/**
 * PumaInvKin.java
 *
 * A class to encapsulate the inverse kinematics for the Unimation 560 Puma robot.
 *
 * @author Brendon Wilson
 * @version alpha
 */
public class PumaInvKin extends InvKin
{
    /**
     * Default constructor.
	 */
    protected PumaInvKin()
    {
        super();
    }

    /**
     * Returns the closest solution in JointVariables form, if a solution 
     * exists, otherwise returns null. Modified from the equations given in 
     * Craig, "Introduction to Robotics".  Note that the desired pitch angle 
     * is implicitly equal to the current pitch, and the desired roll angle is
     * the current roll.
     *
     * @param target the target Point to use to find the solution.
     * @returns the closest solution or null.
     * @exception OutOfRangeException raised when the requested point has no 
     *            solutions, and is out of range.
     */
    public JointVariables getJointVariables (Point3d target)
        throws OutOfRangeException
    {
        
        
        // old stuff...
        // Check that target is within x-z plane.
        if (target.y != 0)
        {
            throw new OutOfRangeException();
        }
        
        double x, y, c2, s2plus, s2minus, k1, k2plus, k2minus;
        JointVariables nearest = (JointVariables)getJointVariables().clone();
        JointVariables nearest2 = (JointVariables)getJointVariables().clone();
        Degree degree;
        
        // Transform coordinates to match model.
        x = target.x;
        y = target.z;
        
        c2 =  (Math.pow(x, 2) + Math.pow(y, 2) - Math.pow(getJointVariables().getJoint("ELBOW").getLength(), 2) - Math.pow(getJointVariables().getJoint("SHOULDER").getLength(), 2))/(2*getJointVariables().getJoint("ELBOW").getLength()*getJointVariables().getJoint("SHOULDER").getLength());

        if (Math.abs(c2) > 1)
        {
            // Solution outside workspace, therefore no solution exists.
            throw  new OutOfRangeException();
        }

        s2plus =  Math.sqrt(1 - Math.pow(c2, 2));
        s2minus =  -s2plus;
        
        // Solve for theta2 - Elbow Angle.
        degree =  nearest.getDegree("ELBOW");
        degree.setVariable(TO_DEGREES*Math.atan2(s2plus, c2));
        degree =  nearest2.getDegree("ELBOW");
        degree.setVariable(TO_DEGREES*Math.atan2(s2minus, c2));
        
        // Solve for theta1 - Shoulder angle 
        k1 = getJointVariables().getJoint("ELBOW").getLength() + getJointVariables().getJoint("SHOULDER").getLength()*c2;
        k2plus = getJointVariables().getJoint("SHOULDER").getLength()*s2plus;
        k2minus = getJointVariables().getJoint("SHOULDER").getLength()*s2minus;
        nearest.getDegree("SHOULDER").setVariable(TO_DEGREES*(Math.atan2(y, x) - Math.atan2(k2plus, k1)));
        nearest2.getDegree("SHOULDER").setVariable(TO_DEGREES*(Math.atan2(y, x) - Math.atan2(k2minus, k1)));

        // Solve for theta3 - Pitch angle.
        degree = nearest.getDegree("PITCH");
        degree.setVariable(getJointVariables().getDegree("PITCH").getVariable() - nearest.getDegree("SHOULDER").getVariable() - nearest.getDegree("ELBOW").getVariable());
        degree = nearest2.getDegree("PITCH");
        degree.setVariable(getJointVariables().getDegree("PITCH").getVariable() - nearest2.getDegree("SHOULDER").getVariable() - nearest2.getDegree("ELBOW").getVariable());

        if (nearest.IsInRange() && nearest2.IsInRange())
        {
            // Both set of joint variables are solutions. Find out which one is the nearest 
            // by taking sum of difference of squares.
            if ((Math.pow(getJointVariables().getDegree("SHOULDER").getVariable() - nearest.getDegree("SHOULDER").getVariable(), 2) + Math.pow(getJointVariables().getDegree("ELBOW").getVariable() - nearest.getDegree("ELBOW").getVariable(), 2) + Math.pow(getJointVariables().getDegree("PITCH").getVariable() - nearest.getDegree("PITCH").getVariable(), 2)) <= (Math.pow(getJointVariables().getDegree("SHOULDER").getVariable() - nearest2.getDegree("SHOULDER").getVariable(), 2) + Math.pow(getJointVariables().getDegree("ELBOW").getVariable() - nearest2.getDegree("ELBOW").getVariable(), 2) + Math.pow(getJointVariables().getDegree("PITCH").getVariable() - nearest2.getDegree("PITCH").getVariable(), 2)))
            {
                return nearest; // nearest is closest solution.
            }
            else
            {
                return nearest2; // nearest2 is closest solution.
            }
        }
        else
        {
            // One of them is not a solution, because it's out of range.
            if (nearest.IsInRange())
            {
                return nearest;
            }
            else if (nearest2.IsInRange())
            {
                return nearest2;
            }

            // Both solutions outside workspace, therefore no solution exists.
            throw new OutOfRangeException();
        }
    }

    /**
     * Implements the clone() method, returning an Object copy of this object.
     *
     * @return an Object copy of this JointVariables.
     */
    public Object clone()
    {
        JointVariables newJointVariables = (JointVariables) getJointVariables().clone();
        ScorbotInvKin newInvKin = new ScorbotInvKin();
        newInvKin.setJointVariables(newJointVariables);

        return (Object) newInvKin;
    }
}