/*
 * Robot.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;


/**
 * A class to encapsulate the information and operations required to represent
 * and manipulate a model of a robot.  The Robot class is an implementation of 
 * the Singleton Pattern, as shown in Gamma et al (p. 127), which ensures only
 * one instance of the Robot class is created, and provides access to that 
 * instance.
 *
 * @author Brendon Wilson
 * @version alpha
 */
public class Robot
{
   /**
    * Constant to define the X-axis as the "up" direction. 
    */
    public static final int X_UP =  0;
    
    /**
     * Constant to define the Y-axis as the "up" direction.
     */
    public static final int Y_UP =  1;
    
    /**
     * Constant to define the Z-axis as the "up" direction.
     */
    public static final int Z_UP =  2;

    /**
     * The unique instance of the Robot class.
     */
    private static Robot uniqueInstance;

    /**
     * The current speed for the Robot's motion.
     */
    private double speed;

    /**
     * The observable object which provides JointVariables updates.
     */
    private RemoteObservable theObservable;
    
    /**
     * The object to provide inverse kinematics calculations.
     */
    private InvKin theInvKin;
    
    /**
     * The set of joints for the Robot.
     */
    private JointVariables theJointVariables;
    
    /**
     * Defines which direction is initially "up", point to top of screen.
     */
    private int upAxis;
    
    /**
     * Defines an initial rotation around the "up" axis.
     */
    private double rotation;
    
    /**
     * Defines an initial zoom factor.
     */
    private double zoomFactor;


    /**
     * The private constructor, which creates an instance of the Robot class. 
     * This method is called by the <code>getInstance</code> method to lazy
     * initialize the uniqueInstance variable.
     */
    private Robot()
    {
        super();

        theJointVariables = new JointVariables();
        speed = 5;
    }

    /**
     * A static accessor to provide access to the singleton instance of the 
     * Robot class.
     */
    public static Robot getInstance()
    {
        if (uniqueInstance == null)
        {
            uniqueInstance = new Robot();
        }

        return uniqueInstance;
    }

    /**
     * Loads the simulation configuration file from the Reader using configured
     * RobotParser to parse the stream to configure the Robot object for the 
     * simulation.
     *
     * @param r the Reader stream for the robot configuration file.
     * @exception RobotParserException indicates error parsing configuration.
     */
    public void loadRobotModel(String robotparser, Reader r) throws RobotParserException
    {
        RobotParser theRobotParser;

        try
        {
            theRobotParser = RobotParser.loadParser(robotparser);
            theRobotParser.read(r);
        }
        catch (RobotParserException mre)
        {
            theRobotParser = RobotParser.loadParser(RobotParser.DEFAULT_PARSER);
            theRobotParser.read(r);
        }
    }

    /**
	 * Sets the axis defined as the "up" direction.
	 *
	 * @param upAxis the "up" direction; either X_UP, Y_UP, or Z_UP.
	 */
    public void setUpAxis(int upAxis)
    {
        this.upAxis = upAxis;
    }

    /**
	 * Returns the axis defined as the "up" direction.
	 *
	 * @return the "up" direction; either X_UP, Y_UP, or Z_UP.
	 */
    public int getUpAxis()
    {
        return upAxis;
    }

    /**
	 * Sets the angle of rotation around the "up" axis.
	 *
	 * @param the angle of rotation.
	 */
    public void setRotation(double rotation)
    {
        this.rotation = rotation;
    }

    /**
	 * Returns the angle of rotation around the "up" axis.
	 *
	 * @return the angle of rotation.
	 */
    public double getRotation()
    {
        return rotation;
    }

    /**
	 * Sets the initial zoom factor.
	 *
	 * @param zoomFactor the view zoom scaling factor.
	 */
    public void setZoom(double zoomFactor)
    {
        this.zoomFactor = zoomFactor;
    }

    /**
	 * Returns the initial zoom factor.
	 *
	 * @return the view zoom factor.
	 */
    public double getZoom()
    {
        return zoomFactor;
    }

    /**
	 * Sets the RemoteObservable Object.
	 *
	 * @param theObservable the Observable object supplying updates.
	 */
    public void setObservable(RemoteObservable theObservable)
    {
        this.theObservable = theObservable;
    }

    /**
	 * Gets the RemoteObservable Object.
	 *
	 * @return the Observable object supplying updates.
	 */
    public RemoteObservable getObservable()
    {
        return theObservable;
    }

    /**
	 * Sets the JointVariables object to use for the Robot.
	 *
	 * @param theJointVariables the Joint Variables object to use.
	 */
    public void setJointVariables (JointVariables theJointVariables)
    {
        this.theJointVariables = theJointVariables;
        theInvKin.setJointVariables(theJointVariables);
    }

    /**
	 * Returns the current JointVariables object.
	 *
	 * @return the current JointVariable object.
	 */
    public JointVariables getJointVariables()
    {
        return theJointVariables;
    }

    /**
	 * Sets the InvKin object for the Robot.
	 *
	 * @param theInvKin the InvKin object to use for the Robot.
	 */
    public void setInvKin (InvKin theInvKin)
    {
        this.theInvKin = theInvKin;
    }

    /**
     * Moves the Robot to the target set of JointVariables, using an abstract 
     * interpolation algorithm.
     *
     * @param destination the destination for the Robot JointVariables.
     */
    protected void moveToDestination(JointVariables destination)
    {
        Interpolator interpolator = new Interpolator(destination);
        interpolator.start();

        try
        {
            interpolator.join();
        }
        catch (InterruptedException e)
        {

        }
    }

    /**
     * Moves the Robot to its 'home' position.
     */
    public void home()
    {
        JointVariables target = (JointVariables) getJointVariables().clone();
        target.home();

        moveToDestination(target);
    }

    /**
     * Moves a specific joint to a given joint position.
     *
     * @param name the name of the joint to move.
     * @param variable the destination joint variable.
     * @exception thrown if position outside robot's jointspace.
     */
    public void moveJoint(String name, double variable) throws OutOfRangeException
    {
        JointVariables target = (JointVariables) getJointVariables().clone();
        IndexPair theParams = (IndexPair) target.getNameTable().get(name);
        Joint currentJoint = target.getJoint(theParams.joint);
        Degree currentDegree = currentJoint.getDegree(theParams.degree);

        if (currentDegree.inRange(variable))
        {
            currentDegree.setVariable(variable);
        }
        else
        {
            throw new OutOfRangeException();
        }
        
        moveToDestination(target);
    }

    /**
     * Moves the end-effector to a specific x, y, z coordinate.
     *
     * @param x the destination x-coordinate.
     * @param y the destination y-coordinate.
     * @param z the destination z-coordinate.
     */
    public void moveToXYZ(double x, double y, double z) throws OutOfRangeException
    {
        Point3d targetPoint = new Point3d(x, y, z);

        JointVariables target = theInvKin.getJointVariables(targetPoint);
        moveToDestination(target);
    }

    /**
     * Sets the speed of the robot's motion.
     *
     * @param speed the new speed for the robot's motion.
     */
    public void setSpeed(double speed)
    {
        this.speed = speed;
    }

    /**
     * Returns the speed of the robot's motion.
     *
     * @return the current speed for the robot's motion.
     */
    public double getSpeed()
    {
        return speed;
    }

    /**
	 * Returns the robot BranchGroup, assembled using the DH parameters.
	 *
	 * @return the BranchGroup which describe the Robot's physical appearance.
	 */
    public BranchGroup getModels()
    {
        BranchGroup theModels = new BranchGroup();
        Group currentModel = new Group();
        Joint currentJoint = new Joint();
        Joint previousJoint = new Joint();
        Degree currentDegree = new Degree();
        Degree previousDegree = new Degree();
        TransformGroup currentGroup = new TransformGroup();
        TransformGroup previousGroup = new TransformGroup();
        Transform3D theTransform = new Transform3D();
        Transform3D tmpTransform = new Transform3D();
        Matrix3d rotMatrix = new Matrix3d();
        Vector3d transVector = new Vector3d();

        theModels.setCapability(BranchGroup.ALLOW_DETACH);

        // Iterate through each of the Robot's Joint objects.
        for (int i =  1; i <= getJointVariables().countJoints(); i++)
        {
            // Initialize variables.
            theTransform = new Transform3D();
            tmpTransform = new Transform3D();
            rotMatrix = new Matrix3d();
            
            // Get the next Joint and its associated model.
            currentJoint = getJointVariables().getJoint(i);
            currentModel = currentJoint.getModel();

            // Create the link length TransformGroup.
            if (!previousJoint.isEmpty())
            {
                tmpTransform.setTranslation(new Vector3d(previousJoint.getLength(), 0, 0));
                theTransform.mul(tmpTransform);
                currentGroup = new TransformGroup();
                currentGroup.setTransform(theTransform);
                previousGroup.addChild(currentGroup);
                previousGroup = currentGroup;
                theTransform = new Transform3D();
                tmpTransform = new Transform3D();
            }

            // Iterate through the Joint's Degree objects.
            for (int j =  1; j <= currentJoint.countDegrees(); j++)
            {
                // Get the Degree object.
                currentDegree = currentJoint.getDegree(j);
                
                // Incorpate link offset into Transform3D object.  Wrong place!
                // tmpTransform.setTranslation(new Vector3d(0, 0, currentDegree.getOffset()));
                // theTransform.mul(tmpTransform);
                // tmpTransform = new Transform3D();

                // Incorporate link twist into Transform3D object.
                if (!previousDegree.isEmpty())
                {
                    rotMatrix = new Matrix3d();
                    rotMatrix.rotX(previousDegree.getTwist()*Math.PI/180.0);
                    tmpTransform.setRotation(rotMatrix);
                    theTransform.mul(tmpTransform);
                    tmpTransform = new Transform3D();
                }

                // Incorpate link offset into Transform3D object.
                tmpTransform.setTranslation(new Vector3d(0, 0, currentDegree.getOffset()));
                theTransform.mul(tmpTransform);
                tmpTransform = new Transform3D();

                // Incorporate Joint variable into Transform3D object.
                if (currentDegree.getType() == Degree.TYPE_REVOLUTE)
                {
                    currentGroup = new TransformGroup();
                    currentGroup.setTransform(theTransform);
                    previousGroup.addChild(currentGroup);
                    previousGroup = currentGroup;
                    tmpTransform = new Transform3D();
                    theTransform = new Transform3D();
                    rotMatrix = new Matrix3d();
                    rotMatrix.rotZ(currentDegree.getVariable()*Math.PI/180.0);
                    tmpTransform.setRotation(rotMatrix);
                    theTransform.mul(tmpTransform);
                    currentGroup = new TransformGroup();
                    currentGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
                    currentGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
                    currentGroup.setTransform(theTransform);
                    previousGroup.addChild(currentGroup);
                    currentDegree.setTransformGroup(currentGroup);
                    previousGroup = currentGroup;
                    tmpTransform = new Transform3D();
                    theTransform = new Transform3D();
                }
                else if (currentDegree.getType() == Degree.TYPE_PRISMATIC)
                {
                    currentGroup = new TransformGroup();
                    currentGroup.setTransform(theTransform);
                    previousGroup.addChild(currentGroup);
                    previousGroup = currentGroup;
                    tmpTransform = new Transform3D();
                    theTransform = new Transform3D();
                    tmpTransform.setTranslation(new Vector3d(0, 0, currentDegree.getVariable()));
                    theTransform.mul(tmpTransform);
                    currentGroup = new TransformGroup();
                    currentGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
                    currentGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
                    currentGroup.setTransform(theTransform);
                    previousGroup.addChild(currentGroup);
                    currentDegree.setTransformGroup(currentGroup);
                    previousGroup = currentGroup;
                    tmpTransform = new Transform3D();
                    theTransform = new Transform3D();
                }
                else // Degree.TYPE_INANIMATE
                {
                    currentGroup = new TransformGroup();
                    currentGroup.setTransform(theTransform);
                    previousGroup.addChild(currentGroup);
                    previousGroup = currentGroup;
                    tmpTransform = new Transform3D();
                    theTransform = new Transform3D();
                }

                previousDegree = currentDegree;
            }

            // Create Joint TransformGroup object.
            currentGroup = new TransformGroup();
            currentGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
            currentGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
            currentGroup.setTransform(theTransform);
            currentGroup.addChild(currentModel);

            // Add the Group to the super-graph.
            if (!previousJoint.isEmpty())
            {
                previousGroup.addChild(currentGroup);
            }
            else
            {
                theModels.addChild(currentGroup);
            }

            previousJoint = currentJoint;
            previousGroup = currentGroup;
        }

        return theModels;
    }

    /**
     * A public class to handle moving the Robot between two positions.
     */
    public class Interpolator extends Thread
    {
        /**
         * Stores the destination set of JointVariables.
         */
        private JointVariables destination;

        
        public Interpolator(JointVariables destination)
        {
            super();
            setPriority(Thread.MAX_PRIORITY);

            this.destination = destination;                                               
        }

        /**
	     * Moves the robot smoothly between the current and a successive state.
	     *
	     * @param destination the target JointVariables.
         * @return whether the Robot is in the final position.
	     */
        public void run()
        {
            boolean done;

            do
            {
                TransformGroup theTransformGroup = new TransformGroup();
                Transform3D theTransform = new Transform3D();
                Matrix3d rotMatrix;
                Vector3d transVector;
                double delta = 4;
                done = true;

                for (int i =  1; i <= getJointVariables().countJoints(); i++)
                {
                    Joint currentSourceJoint, currentDestinationJoint;
                    currentSourceJoint = getJointVariables().getJoint(i);
                    currentDestinationJoint = destination.getJoint(i);

                    for (int j = 1; j <= currentSourceJoint.countDegrees(); j++)
                    {
                        Degree currentSourceDegree, currentDestinationDegree;
                        currentSourceDegree = currentSourceJoint.getDegree(j);
                        currentDestinationDegree = currentDestinationJoint.getDegree(j);

                        if (currentSourceDegree.getVariable() != currentDestinationDegree.getVariable())
                        {
                            if (currentSourceDegree.getVariable() < currentDestinationDegree.getVariable())
                            {
                                currentSourceDegree.setVariable(currentSourceDegree.getVariable() + delta);
                                done &= false;

                                if (currentSourceDegree.getVariable() > currentDestinationDegree.getVariable()) 
                                {
                                    currentSourceDegree.setVariable(currentDestinationDegree.getVariable());
                                    done &= true;
                                }
                            }
                            else if (currentSourceDegree.getVariable() > currentDestinationDegree.getVariable())
                            {
                                currentSourceDegree.setVariable(currentSourceDegree.getVariable() - delta);
                                done &= false;

                                if (currentSourceDegree.getVariable() < currentDestinationDegree.getVariable())
                                {
                                    currentSourceDegree.setVariable(currentDestinationDegree.getVariable());
                                    done &= true;
                                }
                            }

                            theTransformGroup = currentSourceDegree.getTransformGroup();
                            theTransform = new Transform3D();

                            if (currentSourceDegree.getType() == Degree.TYPE_REVOLUTE)
                            {
                                rotMatrix = new Matrix3d();
                                rotMatrix.rotZ(currentSourceDegree.getVariable()*Math.PI/180);
                                theTransform.setRotation(rotMatrix);
                            }
                            else if (currentSourceDegree.getType() == Degree.TYPE_PRISMATIC)
                            {
                                transVector =  new Vector3d(0, 0, currentSourceDegree.getVariable());
                                theTransform.setTranslation(transVector);
                            }

                            theTransformGroup.setTransform(theTransform);
                        }
                    }
                }

                theObservable.setJointVariables(getJointVariables());

                try
                {
                    sleep((long)(500/Robot.getInstance().getSpeed()));
                }
                catch (InterruptedException ie)
                {

                }

                yield();
            } while(!done);
        }
    }
}