/*
 * RobotParser.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.*;
import java.util.*;

/**
 * An abstract class to encapsulate the functionality required to properly
 * parse a robot configuration file, and construct the robot's internal
 * representation, including a 3D representation.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 *
 * @see Robot
 */
public abstract class RobotParser
{
    /** The default RobotParser class. */
    public final static String DEFAULT_PARSER = "DefaultRobotParser";


    /**
     * Protected constructor. Use <code>loadParser</code> to load a specific 
     * RobotParser instance instead of instantiating a RobotParser directly.
     */
    protected RobotParser()
    {
        super();
    }

    /**
     * Parses the Reader for configuration information.
     *
     * @param r the Reader to parse for configuration information.
     * @exception RobotParserException indicates an error parsing the Reader.
     */
    public abstract void read(Reader reader) throws RobotParserException;

    /**
     * Returns the Robot object created from parsing the Reader stream.
     *
     * @return the Robot object.
     */
    public abstract Robot getRobot();

    /**
     * Loads the RobotParser class specified by the given String.
     *
     * @param s the name of the RobotParser class to be loaded.
     * @return an instance of the requested RobotParser class.
     * @exception RobotParserException indicates an error loading the class.
     */
    public static final RobotParser loadParser(String s)
        throws RobotParserException
    {
        Class aClass;
        RobotParser aRobotParser;

        try
        {
            aClass = Class.forName(s);
            aRobotParser = (RobotParser) aClass.newInstance();

            return aRobotParser;
        }
        catch (Exception e)
        {
            throw new RobotParserException(e.getMessage());
        }
    }
}