/*
 * SimulationEngine.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.vecmath.*;

/**
 * The SimulationEngine coordinates reading Commands via the CommandParser and
 * feeding those Commands to the Robot model.
 *
 * @author Brendon Wilson
 * @version alpha
 */
public class SimulationEngine implements Runnable
{
    /** The CommandParser used to read Commands. */
    protected CommandParser theCommandParser;

    /** The CommandQueue of Commands to be processed. */
    protected CommandQueue theCommandQueue;

    /** A flag indicating a request to stop the simulation. */
    protected boolean requestStop = false;

    /** The Thread controlling the simulation. */
    protected Thread simulationThread;


    /**
     * Constructor, initialize the variables.
     */
    public SimulationEngine()
    {
        super();

        theCommandQueue = new CommandQueue();
        simulationThread = new Thread(this);
    }

    /**
     * Sets the current CommandParser object.
     *
     * @param theCommandParser the new CommandParser object.
     */
    public void setCommandParser(CommandParser theCommandParser)
    {
        this.theCommandParser = theCommandParser;
    }

    /**
     * Parse the given Reader stream for Commands using the current 
     * CommandParser, and load the CommandQueue object for the simulation.
     *
     * @param r the Reader stream to parse for simulation commands.
     * @exception SimulationEngineException indicates error parsing Reader.
     */
    public void loadCommandQueue(Reader r) throws SimulationEngineException
    {
        theCommandQueue = new CommandQueue();

        try
        {
            theCommandParser.loadSimulation(r);
            theCommandQueue = theCommandParser.getCommandQueue();
        }
        catch (CommandParserException cpe)
        {
            throw new SimulationEngineException(cpe.getMessage());
        }
    }

    /**
     * Gets the next Command object from the CommandQueue, and feeds the
     * Command to the Robot to get the destination Robot state. The Robot is 
     * then incrementally moved from the current Robot to the destination state
     * until the destination state is reached.
     *
     * @exception Exception indicates error in simulation.
     */
    public void step() throws Exception
    {
        Command theCommand = theCommandQueue.getCommand();
        theCommand.Execute();
    }

    /**
     * Starts the SimulationEngine's Thread with NORMAL_PRIORITY.
     */
    public void start()
    {
        if (!simulationThread.isAlive())
        {
            simulationThread = new Thread(this);
            requestStop = false;
            simulationThread.start();
        }
    }

    /**
     * The loop for the SimulationEngine Thread, which steps the simulation
     * to the next state while the CommandQueue has more Commands to process.
     * Once the CommandQueue is empty, the SimulationEngine Thread run() loop
     * is exited and the Thread is stopped.
     *
     * Note: This method still needs to be able to raise an exception.
     */
    public void run()
    {
        while (!theCommandQueue.isEmpty() && !requestStop)
        {
            try
            {
                step();
            }
            catch (Exception e)
            {
                requestStop = true;
                JOptionPane.showMessageDialog(null, "The requested point lies outside the robot's workspace.", "Command Outside Workspace", JOptionPane.INFORMATION_MESSAGE);

                break;
            }
        }
    }

    /**
     * Stops the simulation.  Note that this method is NOT meant to implement
     * Runnable.stop(), as that method has been deprecated in JDK 1.2.  Instead,
     * this method properly sets a boolean value which causes the run() loop to
     * exit, allows the Thread to terminate properly.
     */
    public void stop()
    {
        requestStop = true;
    }

    /**
     * Indicates if the simulation has finished.
     *
     * @return indicates is simulation is complete.
     */
    public boolean isComplete()
    {
        return (theCommandQueue.isEmpty() || requestStop);
    }

    /**
     * Returns if the CommandQueue has been loaded and is not empty.
     *
     * @return indicates if the CommandQueue is loaded and not empty.
     */
    public boolean isReady()
    {
        return !theCommandQueue.isEmpty();
    }
}