/*
 * XYZTable.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.util.Hashtable;
import javax.vecmath.Point3d;


/**
 * A class to store named endpoints.  The XYZTable class is an implementation 
 * of the Singleton Pattern, as shown in Gamma et al (p. 127), which ensures 
 * only one instance of the class is created, and provides access to that
 * instance.
 *
 * @author Brendon Wilson
 * @version alpha
 */
public class XYZTable
{
    /**
     * Stores the unique instance of the XYZTable singleton.
     */
    private static XYZTable uniqueInstance;

    /**
     * A Hashtable to store XYZ endpoint data.
     */
    private Hashtable theXYZTable;


    /**
     * Creates a new XYZ table.  This method is private in order to force 
     * objects to use <code>getInstance</code> to obtain the static instance 
     * of the XYZTable.
     */
    private XYZTable()
    {
        theXYZTable = new Hashtable();
    }

    /**
     * Puts a new Point3d into the table, using the given name.
     *
     * @param name the name for the endpoint.
     * @param endpoint the endpoint object for the XYZ endpoint coordinates.
     */
    public void putNamedEndpoint(String name, Point3d endpoint)
    {
        theXYZTable.put(name, endpoint);
    }

    /**
     * Returns the Point3d with the given name from the table.
     *
     * @param name the name of the endpoint to retrieve.
     * @return the named endpoint coordinate object.
     */
    public Point3d getNamedEndpoint(String name)
    {
        return (Point3d) theXYZTable.get(name);
    }

    /**
     * Returns the unique instance of the XYZTable class. The actual instance
     * isn't initialized until <code>getInstance</code> is called by another
     * object.
     *
     * @return the only instance of the XYZTable.
     */
    public static XYZTable getInstance()
    {
        if (uniqueInstance == null)
        {
            uniqueInstance = new XYZTable();
        }

        return uniqueInstance;
    }
}